require 'rugged'
require 'thor'
require 'yaml'

require_relative 'issues'
require_relative 'pull_requests'
require_relative '../github'

module Github
  class CLI < Thor
    CONFIG_FILE_PATH = File.expand_path('~/.github-cli.yml')

    class << self
      attr_reader :repository, :repository_dir, :config

      def configure_repository(dir)
        return @repository if @repository

        @repository_dir = dir
        begin
          @repository = Github::Repository.new(@repository_dir)
        rescue Github::Repository::InvalidRepositoryDirectory
          parent_dir = File.expand_path('..', @repository_dir)

          if @repository_dir != parent_dir
            @repository_dir = parent_dir
            retry
          end
        end
      end

      def load_config
        @config = YAML.load_file(CONFIG_FILE_PATH)
        @config
      end
    end

    desc 'issues SUBCOMMAND ...ARGS', 'Manage issues'
    subcommand 'issues', Issues

    desc 'prs SUBCOMMAND ...ARGS', 'Manage pull requests'
    subcommand 'prs', PullRequests

    private

    def repository
      self.class.repository
    end
  end
end
