require 'action_view'
require 'active_support/core_ext/object/acts_like'
require 'date'
require 'thor'

require_relative 'cli/shell_helpers'

module Github
  class Issues < Thor
    include ActionView::Helpers::DateHelper
    include Github::CLI::ShellHelpers

    desc 'list', 'List issues'
    option :page, type: :numeric, default: 1
    def list
      issues = Github::CLI.repository.issues(page: options[:page])
      page_count = issues.count_pages == 0 ? 1 : issues.count_pages

      say "Issues for #{Github::CLI.repository.id} (Page #{options[:page]}/#{page_count})"

      title_width = width_for_percent(0.45)
      creator_width = width_for_percent(0.15)
      format_s = "%-8s %-#{title_width}.#{title_width}s %-#{creator_width}s %s"

      say sprintf(format_s, '#', 'Title', 'User', 'Date')
      issues.each do |issue|
        say sprintf(format_s, "##{issue.number}", issue.title, issue.user.login, time_ago_in_words(Date.parse(issue.updated_at)) + ' ago')
      end
    end

    desc 'show NUMBER', 'Show issue detail'
    def show(number)
      issue = Github::CLI.repository.issue(number)

      say "##{number} - #{issue.title}"
      say sprintf("%#{terminal_width}s", "#{issue.user.login} on #{issue.created_at}")
      say hr
      say issue.body + "\n"

      return if issue.comments == 0

      comments = Github::CLI.repository.issue_comments(number)

      comments.each do |comment|
        say hr
        say sprintf("%#{terminal_width}s", "#{comment.user.login} on #{comment.created_at}")
        say comment.body
      end
    end
  end
end
